// Import all DSS packages for the Debug Server
import com.ti.ccstudio.scripting.environment.*;
import com.ti.debug.engine.scripting.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.lang.*;
import java.io.*;


public class CommandServer extends Thread
{
	private boolean QUIET = false;

	private TcpServer tcpServer;
	private HashMap<String, Commands> commandMap;
	private JFrame myFrame;
	private TextArea myText;
	String portNumber; 
	int sessionCount;
	
	private void appendText(String text)
	{
		if(!QUIET)
		{
			String currentText = myText.getText();
			if(currentText.length() > 4000)
			{
				currentText = currentText.substring(2000); 
			}
			currentText = currentText + text; 
			myText.setText(currentText);
			myText.append("");
		}
	}
		
	private ScriptingEnvironment scriptEnviron;
	private DebugServer debugServer;
	private DebugSession[] debugSession;

	private boolean close;
	
	private enum Commands
	{
		OPEN,
		CONNECT,
		CONNECT_ALL,
		DISCONNECT,
		DISCONNECT_ALL,
		RESET,
		RESET_ALL,
		LOAD,
		LOAD_ALL,
		RUN,
		RUN_ALL,
		RUN_ASYNCH,
		RUN_ASYNCH_ALL,
		HALT,
		HALT_ALL,
		IS_HALTED,
		IS_CONNECTED,
		RESTART,
		RESTART_ALL,
		WAIT_HALT,
		GET_SYMBOL_ADDRESS,
		READ,
		WRITE,
		ADD_BREAKPOINT_AT_ADDRESS,
		ADD_BREAKPOINT_AT_LINE,
		REMOVE_BREAKPOINT,
		CLOSE,
		EVALUATE,
		MEMORY_MAP,
		MEMORY_MAP_ALL,
		READ_REGISTER,
		WRITE_REGISTER,
		RELOAD_PROGRAM,
		RELOAD_PROGRAM_ALL,
		TERMINATE,
		TERMINATE_ALL,
		STOP,
		INVALID
	}
	
	public CommandServer(int portNumber, boolean quietFrame)
	{
		QUIET = quietFrame; 
		
		this.portNumber = Integer.toString(portNumber);
		
		if(!QUIET)
		{
			myFrame = new JFrame("DSS TCP/IP Server on Port: " + portNumber);
			myText = new TextArea(20, 60);
			myFrame.getContentPane().add(myText);
			myFrame.pack();
			myFrame.setVisible(true);
		}
		
		close = false;

		tcpServer = new TcpServer(portNumber);
		
		// Create our scripting environment object - which is the main entry point into any script and
        // the factory for creating other Script-able Servers and Sessions
        scriptEnviron = ScriptingEnvironment.instance();
		
		commandMap = new HashMap<String, Commands>();
		commandMap.put("open", Commands.OPEN);
		commandMap.put("connect", Commands.CONNECT);
		commandMap.put("connect_all", Commands.CONNECT_ALL);
		commandMap.put("disconnect", Commands.DISCONNECT);
		commandMap.put("disconnect_all", Commands.DISCONNECT_ALL);
		commandMap.put("reset", Commands.RESET);
		commandMap.put("reset_all", Commands.RESET_ALL);
		commandMap.put("load", Commands.LOAD);
		commandMap.put("load_all", Commands.LOAD_ALL);
		commandMap.put("run", Commands.RUN);
		commandMap.put("run_all", Commands.RUN_ALL);
		commandMap.put("run_asynch", Commands.RUN_ASYNCH);
		commandMap.put("run_asynch_all", Commands.RUN_ASYNCH_ALL);
		commandMap.put("halt", Commands.HALT);
		commandMap.put("halt_all", Commands.HALT_ALL);
		commandMap.put("is_halted", Commands.IS_HALTED);
		commandMap.put("is_connected", Commands.IS_CONNECTED);
		commandMap.put("restart", Commands.RESTART);
		commandMap.put("restart_all", Commands.RESTART_ALL);		
		commandMap.put("wait_halt", Commands.WAIT_HALT);
		commandMap.put("get_symbol_address", Commands.GET_SYMBOL_ADDRESS);
		commandMap.put("read", Commands.READ);
		commandMap.put("write", Commands.WRITE);
		commandMap.put("add_breakpoint_at_address", Commands.ADD_BREAKPOINT_AT_ADDRESS);
		commandMap.put("add_breakpoint_at_line", Commands.ADD_BREAKPOINT_AT_LINE);
		commandMap.put("remove_breakpoint", Commands.REMOVE_BREAKPOINT);
		commandMap.put("close", Commands.CLOSE);
		commandMap.put("evaluate", Commands.EVALUATE);
		commandMap.put("memory_map", Commands.MEMORY_MAP);
		commandMap.put("memory_map_all", Commands.MEMORY_MAP_ALL);		
		commandMap.put("read_register", Commands.READ_REGISTER);
		commandMap.put("write_register", Commands.WRITE_REGISTER);
		commandMap.put("reload_program", Commands.RELOAD_PROGRAM);
		commandMap.put("reload_program_all", Commands.RELOAD_PROGRAM_ALL);		
		commandMap.put("terminate", Commands.TERMINATE);
		commandMap.put("terminate_all", Commands.TERMINATE_ALL);
		commandMap.put("stop", Commands.STOP);
	}
	
	public void run()
	{
		while(true)
		{
			String command = tcpServer.getCommand();
			System.out.println("received command: " + command);
			appendText("received command: " + command + "\n");
			String response = executeCommand(command);
			tcpServer.sendResponse(response);
			System.out.println("sent response: " + response);
			appendText("    sent response: " + response + "\n");

			if(close) close(); 
		}
	}
	
	private String executeCommand(String message)
	{
		message = message.trim(); // Remove trailing & leading white space
				
		String[] temp = message.split("\""); // Split message at double quote matches
		String[] params = new String[temp.length / 2];
		
		for(int iter = 0; iter < params.length; iter++)
		{
			params[iter] = temp[(iter * 2) + 1];
		}		
		
		String command = params[0];
		command = command.trim().toLowerCase(); // Remove trailing & leading white space, convert to lower case
		
		// All commands without parameter core should be added to this if list
		if(!
		(command.equals("open")||
		command.equals("close")||
		command.equals("stop")||
		command.equals("connect_all")||
		command.equals("disconnect_all")||
		command.equals("reset_all")||
		command.equals("load_all")||
		command.equals("run_all")||
		command.equals("run_asynch_all")||
		command.equals("halt_all")||
		command.equals("restart_all")||
		command.equals("reload_program_all")||
		command.equals("memory_map_all")||
		command.equals("terminate_all")
		)){
			if(Integer.parseInt(params[1])>(sessionCount-1)) return formatResponse(false, "0", "can only access a previously created session...");
		}
	
		Commands resolvedCommand = commandMap.get(command); // Retrieve command enum from map
		if(resolvedCommand==null) resolvedCommand = Commands.INVALID;
		
		try 
		{
			System.out.println("Executing command: " + resolvedCommand);
			switch(resolvedCommand)
			{						
				case OPEN:
					// params[1] = Environment Timeout (Integer)
					// params[2] = Target Configuration File Path (String)
					// params[3] = Board Regular Expression (String)
					// params[4...] = Cpu Regular Expression List (String)

					//Set environment timeout
					scriptEnviron.setScriptTimeout(Integer.parseInt(params[1]));
					//Create a log file to log script execution
					scriptEnviron.traceBegin("dssScriptLog_port_"+portNumber+".xml", "DefaultStylesheet.xsl");
					//Make the log and console file really verbose
					scriptEnviron.traceSetConsoleLevel(TraceLevel.INFO);
					scriptEnviron.traceSetFileLevel(TraceLevel.ALL);		
					//Open debug session/s to core/s					
					debugServer = (DebugServer) scriptEnviron.getServer("DebugServer.1");					
					debugServer.setConfig(params[2]); //Target configuration file
					debugSession = new DebugSession[params.length-4];
					if(params[3].equals("Any Board")) params[3] = "*";
					if((debugSession.length==1)&&(params[4].equals("Any Cpu"))) params[4] = "*";
					for(int core = 0; core < debugSession.length; core++){
						debugSession[core] = debugServer.openSession(params[3], params[core+4]); //Open session/s
						//Set connection properties
						debugSession[core].options.setString("AutoRunToLabelName", "main");
						debugSession[core].options.setBoolean("AutoRunToLabelOnRestart", true);
						debugSession[core].options.setBoolean("AutoRunToLabelOnReset", true);
						debugSession[core].options.setBoolean("ClearBreakpointsOnLoad", true);
						debugSession[core].options.setBoolean("EnableInstalledBreakpoint", false);
						debugSession[core].options.setString("VerifyAfterProgramLoad", "Full verification");						
					}
					
					sessionCount = debugSession.length;

					return formatResponse(true, "", "debug session/s started on board " + debugSession[0].getBoardName() + " at port " + portNumber + "...");
					
				case CONNECT:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.connect();
					return formatResponse(true, "", "core" + params[1] + " connected as " + debugSession[Integer.parseInt(params[1])].getCPUName() + "...");

				case CONNECT_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].target.connect();
					}
					return formatResponse(true, "", "all cores connected, " + sessionCount + " core/s in total...");
					
				case DISCONNECT:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.disconnect();
					return formatResponse(true, "", "core" + params[1] + ", " + debugSession[Integer.parseInt(params[1])].getCPUName() + " disconnected...");

				case DISCONNECT_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].target.disconnect();
					}
					return formatResponse(true, "", "all cores disconnected, " + sessionCount + " core/s in total...");
					
				case RESET:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.reset();
					return formatResponse(true, "", "core" + params[1] + " reset...");

				case RESET_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].target.reset();
					}
					return formatResponse(true, "", "all cores reset, " + sessionCount + " core/s in total...");												
				case LOAD:
					// params[1] = Core (Integer)
					// params[2] = Program File Path (String)
					// return (void)
					debugSession[Integer.parseInt(params[1])].memory.loadProgram(params[2]); 
					return formatResponse(true, "", "program file loaded on core" + params[1] + "...");

				case LOAD_ALL:
					// params[1...] = Program File Paths (String) 
					// return (void)
					if((params.length-1) < sessionCount)
					{
						return formatResponse(false, "", "number of program files is less than number of open sessions...");
					}
					else
					{
						for(int core = 0; core < sessionCount; core++){
							debugSession[core].memory.loadProgram(params[core+1]);
						}
						return formatResponse(true, "", "all program files loaded, " + sessionCount + " program file/s in total...");
					}
					
				case RUN:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.run();
					return formatResponse(true, "", "program ran on core" + params[1] + "...");

				case RUN_ALL:
					// return (void)
					debugServer.simultaneous.run();
					return formatResponse(true, "", "all programs run, " + sessionCount + " program/s in total...");	
					
				case RUN_ASYNCH:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.runAsynch();
					return formatResponse(true, "", "program running on core" + params[1] + "...");

				case RUN_ASYNCH_ALL:
					// return (void)
					debugServer.simultaneous.runAsynch();
					return formatResponse(true, "", "all programs running, " + sessionCount + " program/s in total...");
					
				case HALT:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.halt();
					return formatResponse(true, "", "core" + params[1] + " halted...");

				case HALT_ALL:
					// return (void)
					debugServer.simultaneous.halt();
					return formatResponse(true, "", "all sessions halted, " + sessionCount + " session/s in total...");
					
				case IS_HALTED:
					// params[1] = Core (Integer)
					// return (Boolean)
					boolean is_halted = debugSession[Integer.parseInt(params[1])].target.isHalted();
					return formatResponse(true, "" + bool2string(is_halted), "core" + params[1] + " halted? -> " + is_halted + " ...");
			
				case IS_CONNECTED:
					// params[1] = Core (Integer)
					// return (Boolean)
					boolean is_connected = debugSession[Integer.parseInt(params[1])].target.isConnected();
					return formatResponse(true, "" + bool2string(is_connected),  "core" + params[1] + " connected? -> " + is_connected + " ...");
					
				case RESTART:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.restart();
					return formatResponse(true, "", "program on core" + params[1] + " restarted...");

				case RESTART_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].target.restart();
					}
					return formatResponse(true, "", "all programs on all cores restarted, " + sessionCount + " program/s in total...");
					
				case WAIT_HALT:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].target.waitForHalt();
					return formatResponse(true, "", "core" + params[1] + " halted...");
					
				case GET_SYMBOL_ADDRESS:
					// params[1] = Core (Integer)
					// params[2] = Symbol (String)
					// return (Long)
					long symbol_address = debugSession[Integer.parseInt(params[1])].symbol.getAddress(params[2]);
					return formatResponse(true, "" + symbol_address,  "symbol " + params[2] + " address on core" + params[1] + " ? -> 0x" + 
					Long.toHexString(symbol_address).toUpperCase() + " ...");
					
				case READ:
					// params[1] = Core (Integer)
					// params[2] = Page (Integer)
					// params[3] = Address (Long)
					// params[4] = Type Size (Integer)
					// params[5] = Number of Values (Integer)
					// params[6] = Sign of Results (Boolean)
					// return (Long[])
					int read_page = Integer.parseInt(params[2]);
					long read_addr = Long.parseLong(params[3]);
					int read_type_size = Integer.parseInt(params[4]);
					int read_length = Integer.parseInt(params[5]);
					boolean read_sign = Boolean.parseBoolean(params[6]);
					
					long[] read_data = debugSession[Integer.parseInt(params[1])].memory.readData(read_page, read_addr, read_type_size, read_length, read_sign); 
					String read_data_string = "";
					for(long read_data_temp: read_data)
					{
						read_data_string += read_data_temp + " ";
					}

					if(read_sign) return formatResponse(true, read_data_string, read_type_size + "-bit signed memory read from core" + params[1] + "...");
					else return formatResponse(true, read_data_string, read_type_size + "-bit unsigned memory read from core" + params[1] + "...");
					
				case WRITE:	
					// params[1] = Core (Integer)
					// params[2] = Page (Integer)
					// params[3] = Address (Long)
					// params[4] = Type Size (Integer)
					// params[5...] = Write Data (Long[])
					// return (void)
					int write_page = Integer.parseInt(params[2]);
					appendText("    write_page= " + write_page + "\n");
					long write_addr = Long.parseLong(params[3]);
					appendText("    write_addr= " + write_addr + "\n");
					int write_type_size = Integer.parseInt(params[4]);
					appendText("    write_type_size= " + write_type_size + "\n");
					int write_length = params.length - 5;
					appendText("    write_length= " + write_length + "\n");

					long[] write_data = new long[write_length];
					for(int i = 0; i < write_length; i++)
					{
						write_data[i] = Long.parseLong(params[i + 5]);
					}
					
					debugSession[Integer.parseInt(params[1])].memory.writeData(write_page, write_addr, write_data ,write_type_size);
					return formatResponse(true, "",  write_type_size + "-bit memory was written to core" + params[1] + "...");
					
				case ADD_BREAKPOINT_AT_ADDRESS:
					// params[1] = Core (Integer)
					// params[2] = Address (Long)
					// return (Integer)
					int breakpoint_id;
					breakpoint_id = debugSession[Integer.parseInt(params[1])].breakpoint.add(Long.parseLong(params[2])); 
					if(breakpoint_id==-1) return formatResponse(false, "" + -1,  "breakpoint set failed on core" + params[1] + "...");
					else return formatResponse(true, "" + breakpoint_id, "breakpoint set on core" + params[1] + "...");

				case ADD_BREAKPOINT_AT_LINE:
					// params[1] = Core (Integer)
					// params[2] = Source File (String)
					// params[3] = Line (Integer)
					// return (Integer)
					breakpoint_id = debugSession[Integer.parseInt(params[1])].breakpoint.add(params[2],Integer.parseInt(params[3])); 
					if(breakpoint_id==-1) return formatResponse(false, "" + -1,  "breakpoint set failed on core" + params[1] + "...");
					else return formatResponse(true, "" + breakpoint_id, "breakpoint set on core" + params[1] + "...");					
					
				case REMOVE_BREAKPOINT:
					// params[1] = Core (Integer)
					// params[2] = Breakpoint ID (Integer)
					// return (void)
					if(Integer.parseInt(params[2])==-1){
						debugSession[Integer.parseInt(params[1])].breakpoint.removeAll();
						return formatResponse(true, "", "all breakpoints removed on core" + params[1] + "...");
					}
					else{
						debugSession[Integer.parseInt(params[1])].breakpoint.remove(Integer.parseInt(params[2])); 
						return formatResponse(true, "",  "breakpoint " + params[2] + " removed on core" + params[1] + "...");
					}
					
				case CLOSE:
					close = true;
										
					scriptEnviron.traceEnd(); // Close the log file to prevent memory leak
					return formatResponse(true, "",  "closing tcp session on port "+portNumber+"...");		
					
				case EVALUATE:
					// params[1] = Core (Integer)
					// params[2] = Expression (String)
					// return (Long)
					long evaluationResult = debugSession[Integer.parseInt(params[1])].expression.evaluate(params[2]);
					return formatResponse(true, "" + evaluationResult, "core" + params[1] + " evaluation result of " + params[2] + "? -> " + evaluationResult + " ...");
					
				case MEMORY_MAP:
					// params[1] = Core (Integer)
					// params[2] = State (Boolean)
					// return (void)
					debugSession[Integer.parseInt(params[1])].memory.map.enable(Boolean.parseBoolean(params[2]));
					if(Boolean.parseBoolean(params[2])) return formatResponse(true, "", "memory map enabled on core" + params[1] + "...");
					else return formatResponse(true, "", "memory map disabled on core" + params[1] + "...");
				
				case MEMORY_MAP_ALL:
					// params[1] = State (Boolean)
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].memory.map.enable(Boolean.parseBoolean(params[1]));
					}
					if(Boolean.parseBoolean(params[1])) return formatResponse(true, "", "memory map enabled on all cores, " + sessionCount + " cores in total...");
					else return formatResponse(true, "", "memory map disabled on all cores, " + sessionCount + " core/s in total...");
					
				case READ_REGISTER:
					// params[1] = Core (Integer)
					// params[2] = Register (String)
					// return (Long)
					long register_data = debugSession[Integer.parseInt(params[1])].memory.readRegister(params[2]);
					return formatResponse(true, Long.toString(register_data), "register read on core" + params[1] + "...");
				
				case WRITE_REGISTER:
					// params[1] = Core (Integer)
					// params[2] = Register (String)
					// params[3] = Data (Long)
					// return (void)
					debugSession[Integer.parseInt(params[1])].memory.writeRegister((params[2]), Long.parseLong(params[3]));
					return formatResponse(true, "", "register written on core" + params[1] + "...");
					
				case RELOAD_PROGRAM:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].memory.reloadProgram();
					return formatResponse(true, "", "program on core" + params[1] + " reloaded...");	

				case RELOAD_PROGRAM_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].memory.reloadProgram();
					}
					return formatResponse(true, "", "all programs on all cores reloaded, " + sessionCount + " program/s in total...");										
				case TERMINATE:
					// params[1] = Core (Integer)
					// return (void)
					debugSession[Integer.parseInt(params[1])].terminate();
					return formatResponse(true, "", "debug session for core" + params[1] + " terminated...");

				case TERMINATE_ALL:
					// return (void)
					for(int core = 0; core < sessionCount; core++){
						debugSession[core].terminate();
					}
					return formatResponse(true, "", "all debug sessions terminated, " + sessionCount + " session/s in total...");
					
				case STOP:
					// return (void)
					debugServer.stop();
					return formatResponse(true, "", "debug server terminated...");					
			
				case INVALID:
					return formatResponse(false, "", "Invalid command: " + command);
			}
			
		}
		catch (Exception e)
		{
			exceptionLog(e);
			return formatResponse(false, "0","" + e);
		}
	
		return formatResponse(false, "", "Fell through switch statement?");
	}

	private void close()
	{
		tcpServer.close();
		System.exit(0);
	}
	
	private String bool2string(boolean bool)
	{
		if(bool)
		{
			return 1 + "";
		}
		else
		{
			return 0 + "";
		}
	}
	
	public String formatResponse(boolean success, String data, String comments)
	{
		String temp = "";
		if(success)
		{
			temp += "SUCCESS ";
		}
		else
		{
			temp += "ERROR ";
			close = true; // Close the server to prevent memory leak
			try{
				scriptEnviron.traceEnd(); // Close the log file to prevent memory leak
				if(!QUIET){
					PrintWriter logFile = new PrintWriter("tcpSessionLog_port_"+portNumber+".txt"); // Dump tcp session to file
					logFile.println(myText.getText());
					logFile.close();
				}
			}
			catch (Exception e)
			{
				exceptionLog(e);
				return "ERROR  // "+e;
			}
		}
		temp += data;
		temp += " // " + comments;
		return temp;
	}
	
	public void exceptionLog(Exception e)
	{
		appendText("toSting(): " + e.toString()+ "\n");
		appendText("getLocalizedMessage(): " + e.getLocalizedMessage()+ "\n");
		appendText("getMessage(): " + e.getMessage()+ "\n");
		appendText("getStackTrace(): " + e.getStackTrace() + "\n");	
	}
	
	public static void main(String[] args)
	{
		// args[0] = Port (Integer)
		// args[1] = Quiet Frame (Boolean)
		CommandServer server = new CommandServer(Integer.parseInt(args[0]), Boolean.parseBoolean(args[1]));
		server.start();
	}
}